% Estimator of the Hurst parameter of LRD stationary time series 
% using Wavelet Based Analysis and 
% following an adaptive procedure of Bardet et al. (2008)
%
%
% Input :	* traj: a path of length >140; 
%				* printout:	=1 draw the graph of log-log regression
%								=0 no graph
%
% Output :	* H: estimator of Hurst parameter
%				* alpha: N^alpha is the smallest considered scale
%				* m: number of considered scales for the regression
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [Hbbj,alpha,m]=WaveBBJadaptif(traj,printout);

if nargin<2, 
   printout=0;
end

N=length(traj);
   
if size(traj'*traj)==1
   traj=traj';
end

      TN=0;
      m1=15;
      aN=floor(N.^((1:5*log(N))/(5*log(N))));
      amin=min(find(aN>=3));
      amax1=max(find(aN<(N/(1.8*m1))));
      bmin=min(find(aN>=3*m1));
      bmax=max(find(aN<N/2));


	   d=0;
      d1=0;
      K1=0;
	   K=0;
      Y=0;
      QN=0;
      L=log(1:m1)';     
      for p=amin:(amax1-1)
     	  for i=1:m1
     			 TN(p,i) = TNbbj(i*aN(p),N,traj,0);
        end
   	  Y=log(TN(p,1:m1))-L'-log(aN(p));
   	  [d(p),K(p),M]=estimbbj(L,Y,aN(p));
        QN(p)=((eye(m1)-M*inv(M'*M)*M')*Y')'*((eye(m1)-M*inv(M'*M)*M')*Y');
      end
      Q=min(QN(amin:(amax1-1)));
      aa=max(find(QN==Q));
      alpha=log(aN(aa))/log(N);
      
      for p=bmin:bmax
     	  for i=1:m1
     			 WN(p,i) = TNbbj(floor(aN(p)/i),N,traj,0);
        end
   	  Yb=log(WN(p,1:m1))+L'-log(aN(p));
   	  [db(p),Kb(p),Mb]=estimbbj(-L,Y,aN(p));
        QNb(p)=((eye(m1)-Mb*inv(Mb'*Mb)*Mb')*Yb')'*((eye(m1)-Mb*inv(Mb'*Mb)*Mb')*Yb');
      end
      Q2b=min(QNb(bmin:(bmax-1)));
      bb=max(find(QNb==Q2b));
      bbbbj=log(aN(bb))/log(N);

      Dprime=(1-log(aN(aa))/log(N))/(2*log(aN(aa))/log(N));
            aatilde=min(floor(aa+(15*log(log(N)))/((m1-2)*Dprime)),floor(5*log(N/m1)-2));
      aaprime=log(aN(aatilde))/log(N);
               
      L=0;
      Y=0;
      VN=0;
      m2=floor(aN(bb)/aN(aa));
      amax2=max(find(aN<floor(N/5)));
      m=max(floor(5*N^0.1),min(m2,floor(aN(amax2)/aN(aa))));
      p=aa;
      for i=1:m
     	  VN(i) = TNbbj(i*aN(p),N,traj,0);
      end
      L=log(1:m)';
      Y=log(VN)-L'-log(aN(p));
      [d1,K1,M1]=estimbbj(L,Y,aN(p));
      Hbbj=1-d1/2;
      
      if printout==1
          i=1:m1;
          figure(1)
          clf
          TTN=0;
          ii=0;
          for pp=amax1:amax2
              ii=ii+1;
              TTN(ii)=TNbbj(aN(pp),N,traj,0)
          end
          pp=amax1:amax2;
          TN(pp,1)=TTN';
          p=amin:amax2;
          hold on
          plot(log(aN(p))/log(N),log(TN(p,1)))
          plot(log((1:m)*aN(aa))/log(N),M1*inv(M1'*M1)*M1'*log(VN)','r')
          xlabel('Power of the scale')
          ylabel('Logarithm of the sample variance of wavelet coefficients')
          hold off
          %hold on
          %   for p=amin:(amax1-1)
          %      i=1:m1;
     	    %   plot(log(i*aN(p))/log(N),log(TN(p,i)),'.')
          %end
      end
            
      
      
function [TN] = TNbbj(aN,N,traj,r)

t=0:(aN-1);
e=0;
k=0;
for b=(1+floor(r*N/aN)):(floor((1-r)*N/aN)-1)
    k=k+1;
    %e(k)=(1/sqrt(aN))*sum(psihatemint(t,aN).*traj(t+aN*b));
    e(k)=(1/sqrt(aN))*sum(psiLRD(t/aN).*traj(t+aN*b));
end
TN=1/((1-2*r)*N/aN)*sum(e.^2);

function [y] = psiLRD(t)

%y=1-4*(t<=3/4)+6*(t<=1/2)-4*(t<=1/4);
%y=1-5*(t<=0.8)+10*(t<=0.6)-10*(t<=0.4)+5*(t<=0.2);
%y=100*t.*(t-1).*(t.^2-t+0.2); 
y=100*t.^2.*(t-1).^2.*(t.^2-t+3/14); 
%y=10000*t.^3.*(t-1).^3.*(t.^3-1.5*t.^2+15/22*t-1/11); 
%t(t==0)=0.0000000001;
%t(t==1)=0.9999999999;
%y=10000*exp(-1./(t.*(1-t))).*(t.^2-t+0.23087577);

function [d,K,M]=estimbbj(L,Y,bN);

m=length(L);
Im=ones(m,1);
LL=(L+log(bN));
S=Y';
M=[-LL,-Im];
T1=inv(M'*M)*M'*S;
d=[1 0]*T1;
K=[0 1]*T1;


